#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import with_statement

from ionas.filesync.basefilesync import (
    SrcFull,
    DstFull,
    MODE_CH_FILE,
    MODE_CH_DIR,
    MTIME_NAME,
    STATE_SAME,
    DEBUG,
)
from ionas.exception import *
from ionas.exception import SizeTooLargeError
from homebox2 import (
    BackupHomeBox2Access,
)

import stat


class HomeBox2FileTree(SrcFull, DstFull):

    def __init__(self,
                 path,
                 target_info,
                 # proxy_host=None,
                 # proxy_port=None,
                 get_proxy=lambda url: (None, None),
                 ignored_list=[],
                 log_callback=None,
                 tmp_path="",
                 encoding="",
                 is_skip_file_callback=None,
                 ignore_info_list=[]):

        self._backup_access = BackupHomeBox2Access(target_info, get_proxy)
        # if not path:
        #     path = target_info.subdir

        # if path.endswith("/"):
        #     path = path[:-1]

        SrcFull.__init__(self,
                         path,
                         ignored_list,
                         log_callback,
                         ignore_info_list=ignore_info_list)
        DstFull.__init__(self,
                         path,
                         log_callback,
                         "",
                         is_skip_file_callback=is_skip_file_callback)

        self.MAX_FILE_SIZE = None

        try:
            self._backup_access.stat(path)
        except NoEntryError:
            raise NoRootEntryError

        self.sep = self._backup_access.sep

    @property
    def max_num_transfer_threads(self):
        return 1

    def is_skip_err(self, exc_type, exc_ins):
        skip = DstFull.is_skip_err(self, exc_type, exc_ins)
        skip = skip or isinstance(exc_ins, SizeTooLargeError)
        return skip

    def listdir(self, path):
        return self._backup_access.listdir(path)

    def open(self, path, mode):
        return self._backup_access.open(path, mode)

    def mkdir(self, path):
        self._backup_access.mkdir(path)

    def stat(self, path):
        return self._backup_access.stat(path)

    def chmod(self, path, mode):
        self._backup_access.chmod(path, mode)

    def utime(self, path, mtime):
        self._backup_access.utime(path, mtime)

    def remove(self, path):
        self._backup_access.remove(path)

    def rmdir(self, path):
        self._backup_access.rmdir(path)

    def move(self, src, dst):
        self._backup_access.move(src, dst)

    def post_file_transfer(self, file_relpath, file_info):
        pass

    def post_dir_create(self, dir_relpath, dir_info):
        pass

    def is_misc_skip_err(self, exc_type, exc_ins):
        return True
