#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Do something for uninstalling chbox."""

import os

import iobase.xmldata as xmldata
import iobase.cmdexec as cmdexec
import ionas.request as request
from product_conf import (LIBEXEC_DIR, STOR_IF_TO_VOL, )
from product_conf.devinfo import (VOL_INFO, )
from fal_info import (FAL_SERVICECTRL_PATH, FAL_VOLCTRL_PATH, FAL_CHBOX_PATH,
                            Results, ReqTypes)
from pkgaddon.chbox_config import (SYNC_SERVICE_NAME,)


INTERNAL_VOL = STOR_IF_TO_VOL["internal"][1]
MANUAL_PATH = os.path.join(VOL_INFO[INTERNAL_VOL]["mnt_point"],
                            VOL_INFO[INTERNAL_VOL]["share_root"],
                            "LAN DISK Manual",
                            "hdlh-chbox-manual.pdf")

UNMOUNT_EVENT_XML = '<events><eventinfo category="volume" ' + \
                                'action="unmount" previous="true">' + \
                    '<volume><name>' + INTERNAL_VOL + '</name></volume>' + \
                    '</eventinfo></events>'

VOLUME_QUERY_REQ_XML_TMPL = \
    '<volctrl><volume><name type="volume">%s</name></volume></volctrl>'

VOLUME_STATUS_PATH = "volume/status"
STATUS_MOUNTED = "mounted"


SERVICE_LIST_IN_SHARE_CATEOGRY_XPATH = \
    """categorylist/category[name="share"]/servicelist"""
SERVICELIST_NAME = "servicelist"
SERVICE_NAME = "service"
NAME_NAME = "name"
SERVICECTRL_SERVICE_REQ_XML = """<servicectrl>
    <servicelist>
        <service><name>%s</name></service>
    </servicelist>
</servicectrl>
""" % (SYNC_SERVICE_NAME, )


def __send_request(target, type, body=""):
    """Send request entry to "target".""" 

    entry = request.Entry(type, body)
    return request.Request(target).request(entry).get_entry()


def is_volume_mounted(vol):
    """Check if "vol" is mounted."""

    result_entry = __send_request(FAL_VOLCTRL_PATH, ReqTypes.QUERY,
                            VOLUME_QUERY_REQ_XML_TMPL % (vol,))

    if (result_entry.get_result() == Results.OK and
        result_entry.get().get_value(VOLUME_STATUS_PATH) == STATUS_MOUNTED):

        return True

    return False


def try_stop_chbox_service():
    """Try_stop service for the chbox package."""

    try:
        __send_request(FAL_SERVICECTRL_PATH, ReqTypes.STOP,
                            SERVICECTRL_SERVICE_REQ_XML)
    except:
        # Ignore error
        pass


def delete_chbox_service():
    """Add service for the chbox package to a servicectrl's configuration
       file.
    """

    try:
        servicectrl_config = \
                    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.LOAD).get()

        servicelist_in_category = \
            servicectrl_config.get_node(SERVICE_LIST_IN_SHARE_CATEOGRY_XPATH)
        servicelist = servicectrl_config.get_node(SERVICELIST_NAME)

        is_deleted = False
        for list_node in (servicelist_in_category, servicelist):
            for (idx, service) in \
                    enumerate(list_node.get_node_list(SERVICE_NAME)):

                try:
                    if (service.get_node(NAME_NAME).get_text() ==
                                SYNC_SERVICE_NAME):
                        list_node.delete_child(SERVICE_NAME, index=idx)
                        is_deleted = True
                except:
                    pass

        if is_deleted:
            __send_request(FAL_SERVICECTRL_PATH, ReqTypes.SAVE,
                            servicectrl_config)
    except:
        pass


def uninstall_manual():
    """Uninstall manual for the chbox package by a event handler."""

    if os.path.exists(MANUAL_PATH):
        os.unlink(MANUAL_PATH)


#
# Main
#

try_stop_chbox_service()
delete_chbox_service()

if is_volume_mounted(INTERNAL_VOL):
    uninstall_manual()
