# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _
from ionas.request import Entry, XmlNode
from fal_info import Results
from datetime import datetime
import time


class CHBox(Command):
    try:
        from fal_info import FAL_CHBOX_PATH as COMMAND
    except:
        COMMAND_NAME = "fal_chbox.py"

    ##########################################################################
    # 公開用
    def add(self, job, messages={}, log=True):
        default_messages = {
            Results.OK: ["MSG_CHBOX_ADD", {}],
            None: ["ERR_CHBOX_ADD", {}],
        }
        default_messages.update(messages)

        try:
            entry = self._add_modify(job, mode="add")
            result = entry.get_result() if isinstance(entry, Entry) else None

            message = default_messages.get(result, default_messages[None])
            self._set_message(message[0], arglist=message[1], log=log)

            return result in [Results.OK]
        except:
            return False

    def modify(self, job, messages={}, log=True):
        default_messages = {
            Results.OK: ["MSG_CHBOX_MODIFY", {}],
            None: ["ERR_CHBOX_MODIFY", {}],
        }
        default_messages.update(messages)

        try:
            entry = self._add_modify(job, mode="modify")
            result = entry.get_result() if isinstance(entry, Entry) else None

            message = default_messages.get(result, default_messages[None])
            self._set_message(message[0], arglist=message[1], log=log)

            return result in [Results.OK]
        except:
            return False

    def delete(self, sid, messages={}, log=True):
        default_messages = {
            Results.OK: ["MSG_CHBOX_DELETE", {}],
            None: ["ERR_CHBOX_DELETE", {}],
        }
        default_messages.update(messages)

        try:
            entry = self._delete(sid)
            result = entry.get_result() if isinstance(entry, Entry) else None

            message = default_messages.get(result, default_messages[None])
            self._set_message(message[0], arglist=message[1], log=log)

            return result in [Results.OK]
        except:
            return False

    def list(self, portal=u"", loginid=u"", passwd=u""):

        def __list(entry):
            try:
                folderlist = []
                for node in entry.get().get_node_list("chboxsync/folderlist/folder"):
                    try:
                        folderlist.append({
                            "path": node.get_node("path").get_text(),
                        })
                    except:
                        pass
                return folderlist
            except:
                return []

        entry = self._list(portal, loginid, passwd)
        return __list(entry)

    def load(self):

        def _dict(entry):
            data = {}
            try:

                def __is_enabled(node, xpath=".", default=False):
                    try:
                        return node.get_node(xpath).is_enabled()
                    except:
                        return default

                for share_node in entry.get().get_node_list("sharelist/share"):
                    sid = share_node.get_value("@id")
                    data[sid] = {}
                    data[sid]["sid"] = sid
                    data[sid]["enable"] = __is_enabled(
                            share_node, "shareservice/chboxsync")
                    data[sid]["downloadinterval"] = int(share_node.get_value(
                            "shareservice/chboxsync/downloadinterval"))
                    data[sid]["uploadmaxfilesize"] = int(share_node.get_value(
                            "shareservice/chboxsync/uploadmaxfilesize"))
                    data[sid]["share"] = {}
                    data[sid]["share"]["name"] = share_node.get_value(
                            "name")
                    data[sid]["chbox"] = {}
                    data[sid]["chbox"]["portal"] = share_node.get_value(
                            "shareservice/chboxsync/portalid")
                    data[sid]["chbox"]["loginid"] = share_node.get_value(
                            "shareservice/chboxsync/loginid")
                    data[sid]["chbox"]["passwd"] = share_node.get_value(
                            "shareservice/chboxsync/passwd")
                    data[sid]["chbox"]["folder"] = {}
                    data[sid]["chbox"]["folder"]["path"] = share_node.get_value(
                            "shareservice/chboxsync/folder/path")
            except:
                pass
            return data

        entry = self._load()
        return _dict(entry)

    def query(self, sid):

        def _dict(entry):
            data = {}
            try:

                def __is_enabled(node, xpath=".", default=False):
                    try:
                        return node.get_node(xpath).is_enabled()
                    except:
                        return default

                chbox_node = entry.get()
                data["sid"] = chbox_node.get_node("share").get_attr_value("id")
                data["enable"] = __is_enabled(
                        chbox_node, "share/shareservice/chboxsync")
                data["status"] = chbox_node.get_value(
                        "share/shareservice/chboxsync/status")
                data["downloadinterval"] = int(chbox_node.get_value(
                        "share/shareservice/chboxsync/downloadinterval"))
                data["uploadmaxfilesize"] = int(chbox_node.get_value(
                        "share/shareservice/chboxsync/uploadmaxfilesize"))
                data["share"] = {}
                data["share"]["name"] = chbox_node.get_value(
                        "share/name")
                data["chbox"] = {}
                data["chbox"]["portal"] = chbox_node.get_value(
                        "share/shareservice/chboxsync/portalid")
                data["chbox"]["loginid"] = chbox_node.get_value(
                        "share/shareservice/chboxsync/loginid")
                data["chbox"]["passwd"] = chbox_node.get_value(
                        "share/shareservice/chboxsync/passwd")
                data["chbox"]["folder"] = {}
                data["chbox"]["folder"]["path"] = chbox_node.get_value(
                        "share/shareservice/chboxsync/folder/path")
                data["chbox"]["folder"]["desc"] = chbox_node.get_value(
                        "share/shareservice/chboxsync/folder/desc")
            except:
                pass
            return data

        entry = self._query(sid)
        return _dict(entry)

    def download(self, sid, messages={}, log=True):
        #default_messages = {
        #    Results.OK: ["MSG_OK", {}],  # FIXME msg
        #    None: ["MSG_NG", {}],  # FIXME msg
        #}
        #default_messages.update(messages)

        entry = self._download(sid)
        result = entry.get_result() if isinstance(entry, Entry) else None

        #message = default_messages.get(result, default_messages[None])
        #self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]
                                                                             #
    ##########################################################################

    ##########################################################################
    # 内部用
    def _add_modify(self, job, mode="modify"):
        """
        job = {
            "sid": "123456789",  # NOTE modify only
            "enable": True,
            "downloadinterval": 3600,
            "uploadmaxfilesize": 10,
            "share": {
                "name": "disk1",  # NOTE add only
            },
            "chbox": {
                "portal": "MyPortal",
                "loginid": "abc",
                "passwd": "pa55w0rd",
                "folder": {
                    "path": "/Users/MyFolder",
                },
            },
        }
        return Entry or None
        """
        entry = Entry(
                mode,
                """<chbox><share><shareservice><chboxsync /></shareservice></share></chbox>""")
        try:
            """
            <chbox>
                <share id="1234567890">
                    <name>disk1</name>
                    <shareservice>
                        <chboxsync enable="true">
                            <portalid>1234567890</portalid>
                            <loginid>qwertyuiop</loginid>
                            <passwd>pa55w0rd</passwd>
                            <folder>
                                <path>/Users/MyFolder</path>
                            </folder>
                            <downloadinterval>3600</downloadinterval>
                            <uploadmaxsize>10</uploadmaxsize>
                        </chbox>
                    </shareservice>
                </share>
            </chbox>
            """
            #entry = Entry("add", """<chbox><share /></chbox>""")
            share_node = entry.get().get_node("share")
            if mode == "modify":
                share_node.set_attrs({"id": job["sid"]})
            if mode == "add":
                share_node.append_child("name", text=job["share"]["name"])
            chbox_node = share_node.get_node("shareservice/chboxsync")
            chbox_node.enable(job["enable"])
            #if job["enable"]:
            chbox_node.append_child("loginid", text=job["chbox"]["loginid"])
            chbox_node.append_child("portalid", text=job["chbox"]["portal"])
            chbox_node.append_child("passwd", text=job["chbox"]["passwd"])
            #chbox_node.append_child("folder", text=job["chbox"]["folder"]["path"])
            chbox_node.append_child("folder")
            chbox_node.get_node("folder").append_child("path", text=job["chbox"]["folder"]["path"])
            chbox_node.append_child("downloadinterval", text=str(job["downloadinterval"]))
            chbox_node.append_child("uploadmaxfilesize", text=str(job["uploadmaxfilesize"]))
        except KeyError:
            return None

        return self.request(entry)

    def _delete(self, sid):
        entry = Entry("delete", """<chbox><share /></chbox>""")
        entry.get().get_node("share").set_attrs({"id": sid})
        entry = self.request(entry)
        return entry

    def _list(self, portal, loginid, passwd):
        entry = Entry(u"list", u"""<chbox><chboxsync /></chbox>""")
        chboxsync_node = entry.get().get_node(u"chboxsync")
        chboxsync_node.append_child(u"portalid", text=portal)
        chboxsync_node.append_child(u"loginid", text=loginid)
        chboxsync_node.append_child(u"passwd", text=passwd)
        return self.request(entry)

    def _load(self):
        entry = Entry("load", """<chbox />""")
        return self.request(entry)

    def _query(self, sid):
        entry = Entry("query", """<chbox><share /></chbox>""")
        entry.get().get_node("share").set_attrs({"id": sid})
        entry = self.request(entry)
        return entry

    def _download(self, sid):
        entry = Entry("download", """<chbox><share /></chbox>""")
        entry.get().get_node("share").set_attrs({"id": sid})
        entry = self.request(entry)
        return entry

                                                                             #
    ##########################################################################


if __name__ == '__main__':
    chbox = CHBox()
    print "ADD: ", chbox.add({
        "enable": False,
        "share": {
            "name": "disk1",
        },
    })
    print "ADD: ", chbox.add({
        #"sid": "",
        "enable": True,
        "downloadinterval": 3600,
        "uploadmaxfilesize": 10,
        "share": {
            "name": "disk2",
        },
        "chbox": {
            "portal": "1234567890",
            "loginid": "qwertyuiop",
            "passwd": "pa55w0rd",
            "folder": {
                "path": "MyFolder",
            },
        },
    })
    print "MODIFY: ", chbox.modify({
        "sid": "1234567890",
        "enable": False,
    })
    print "MODIFY: ", chbox.modify({
        "sid": "1234567890",
        "enable": True,
        "uploadmaxfilesize": 10,
        "downloadinterval": 3600,
        #"share": {
        #    "name": "disk2",
        #},
        "chbox": {
            "portal": "1234567890",
            "loginid": "qwertyuiop",
            "passwd": "pa55w0rd",
            "folder": {
                "path": "MyFolder",
            },
        },
    })
