# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponse
from django import forms
from django.core.exceptions import ValidationError
from django.utils import simplejson
from project.view import FuncView, RootView, MenuView
from project.fal.chbox import CHBox
from project.forms.fields.extras.chbox import JobListCheckboxField
from project.forms.fields.extras.chbox import SidHiddenField
from project.forms.fields.extras.chbox import SharenameHiddenField
from project.forms.fields.extras.chbox import SharenameSelectField
from project.forms.fields.extras.chbox import EnableField
from project.forms.fields.extras.chbox import CHBoxBoxIdTextField
from project.forms.fields.extras.chbox import CHBoxLoginidTextField
from project.forms.fields.extras.chbox import CHBoxPortalTextField
from project.forms.fields.extras.chbox import CHBoxPasswordField
from project.forms.fields.extras.chbox import CHBoxFolderTextField
from project.forms.fields.extras.chbox import SyncDirectionRadioField
from project.forms.fields.extras.chbox import IntervalSelectField
from project.forms.fields.extras.chbox import UploadsizeTextField
import os
import re


MAX_JOB_NUMBER = 10


class MenuView(MenuView):
    pass


class ListForm(forms.Form):

    def __init__(self, choices=[], *args, **kwargs):
        super(ListForm, self).__init__(*args, **kwargs)
        self.fields["joblist"] = JobListCheckboxField(
                label=_(u"CHBox: Target Share List"),
                choices=choices,
                required=False)


class ListView(FuncView):

    views = ["delete", "download"]

    def __get_joblist(self,):
        joblist = []
        for config in CHBox().load().values():
            try:
                data = CHBox().query(config["sid"])
                joblist.append({
                    "sid": data["sid"],
                    "enable": data["enable"],
                    "sharename": data["share"]["name"],
                    "status": {
                        "noerror": "o",
                        "fail": "x",
                    }.get(data["status"], "x") if data["enable"] else "-",
                    "interval": data["downloadinterval"],
                    "foldername": data["chbox"]["folder"]["path"],
                })
            except:
                pass
        return joblist

    def __get_joblist_and_form(self, data=None):
        joblist = self.__get_joblist()
        form = ListForm(
                #data=request.POST,
                data=data,
                choices=[(job["sid"], job["sharename"]) for job in joblist])
        for job, cb in zip(joblist, form["joblist"]):
            job["cb"] = cb
        return joblist, form

    def view(self, request):
        joblist, form = self.__get_joblist_and_form()

        overmax = (len(CHBox().load()) >= MAX_JOB_NUMBER)
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "list.html"),
            self.KEY_TEMPLATE_DIC: {
                "joblist": joblist,
                "overmax": overmax,
                "form": form,
            },
        }

    def view_delete(self, request):
        joblist, form = self.__get_joblist_and_form(data=request.POST)

        with self.processing() as p:
            if form.is_valid():
                chbox = CHBox()
                p.result(True)
                if len(form.cleaned_data["joblist"]):
                    for sid in form.cleaned_data["joblist"]:
                        if not chbox.delete(sid):
                            p.result(False)
                            self.set_message(chbox.get_message())
                    if p.result():
                        self.set_message(chbox.get_message())
                else:
                    p.result(False)
                    self.set_message(_(u"CHBox: TargetCount(0)"))
        # NOTE 削除により一覧に変更があるはずなので再生成
        joblist, form = self.__get_joblist_and_form()

        overmax = (len(CHBox().load()) >= MAX_JOB_NUMBER)
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "list.html"),
            self.KEY_TEMPLATE_DIC: {
                "joblist": joblist,
                "overmax": overmax,
                "form": form,
            },
        }

    def view_download(self, request):
        joblist, form = self.__get_joblist_and_form(data=request.POST)

        with self.processing() as p:
            if form.is_valid():
                chbox = CHBox()
                p.result(True)
                if len(form.cleaned_data["joblist"]):
                    for sid in form.cleaned_data["joblist"]:
                        if not chbox.download(sid):
                            p.result(False)
                            sharename = chbox.query(sid).get("share", {}).get("name", "-")
                            self.set_message(_(u"CHBox: Failed to start download. (%(sharename)s)") % {"sharename": sharename})
                    if p.result():
                        self.set_message(_(u"CHBox: Successed to start download."))
                else:
                    p.result(False)
                    self.set_message(_(u"CHBox: TargetCount(0)"))

        overmax = (len(CHBox().load()) >= MAX_JOB_NUMBER)
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "list.html"),
            self.KEY_TEMPLATE_DIC: {
                "joblist": joblist,
                "overmax": overmax,
                "form": form,
            },
        }


class JobForm(forms.Form):

    def __init__(self, mode="add", *args, **kwargs):
        super(JobForm, self).__init__(*args, **kwargs)
        #self.fields["share"] = CHBoxSelectSidField(
        #        label=_(u"CHBox: Target Share Name"),
        #        required=True)
        self.fields["sid"] = SidHiddenField(required=(mode == "modify"))
        self.fields["sharename"] = (SharenameHiddenField if mode == "modify" else SharenameSelectField)(
                label=_(u"CHBox: Target Share Name"),
                required=True)

    enable = EnableField(
            label=_(u"CHBox: CHBox Sync"),
            initial=True,
            required=True)
    boxid = CHBoxBoxIdTextField(
            label=_(u"CHBox: BOX ID"),
            required=False)
    #portal = CHBoxPortalTextField(
    #        label=_(u"CHBox: Portal Name"),
    #        required=False)
    #loginid = CHBoxLoginidTextField(
    #        label=_(u"CHBox: Login ID"),
    #        required=False)
    password = CHBoxPasswordField(
            label=_(u"CHBox: Password"),
            required=False)
    folder = CHBoxFolderTextField(
            label=_(u"CHBox: HOME-BOX Folder"),
            required=False)
    _(u"CHBox: Bidirectionally")
    _(u"CHBox: Upload Only")
    _(u"%(h)d hours") % {"h": 1}
    download = SyncDirectionRadioField(
            label=_(u"CHBox: Sync Direction"),
            required=True)
    interval = IntervalSelectField(
            label=_(u"CHBox: Monitoring Interval"),
            required=False)
    uploadsize = UploadsizeTextField(
            label=_(u"CHBox: Max Upload File Size"),
            required=False)

    def clean(self):
        data = self.cleaned_data
        if data.get("enable", False):
            if data.get("boxid", None) in [""]:
                raise ValidationError("%s: %s" % (
                        _(u"CHBox: BOX ID"),
                        _(u"This field is required.")))
            #if data.get("portal", None) in [""]:
            #    raise ValidationError("%s: %s" % (
            #            _(u"CHBox: Portal Name"),
            #            _(u"This field is required.")))
            #if data.get("loginid", None) in [""]:
            #    raise ValidationError("%s: %s" % (
            #            _(u"CHBox: Login ID"),
            #            _(u"This field is required.")))
            if data.get("password", None) in [""]:
                raise ValidationError("%s: %s" % (
                        _(u"CHBox: Password"),
                        _(u"This field is required.")))
            if data.get("folder", None) in [""]:
                raise ValidationError("%s: %s" % (
                        _(u"CHBox: HOME-BOX Folder"),
                        _(u"This field is required.")))
        return data


class JobView(FuncView):

    mode = None

    views = ['apply']

    def __initial(self, sid):
        if sid in [None, "", [], ()]:
            return {}
        try:
            job = CHBox().query(sid)
            return {
                "sid": job["sid"],
                "sharename": job["share"]["name"],
                "enable": job["enable"],
                "boxid": "%s@%s" % (job["chbox"]["loginid"], job["chbox"]["portal"]),
                #"portal": job["chbox"]["portal"],
                #"loginid": job["chbox"]["loginid"],
                "password": job["chbox"]["passwd"],
                "folder": job["chbox"]["folder"]["path"],
                "download": int(job["downloadinterval"]) > 0,
                "interval": job["downloadinterval"],
                "uploadsize": job["uploadmaxfilesize"],
            }
        except:
            return {}

    def view(self, request, sid=None):
        form = JobForm(initial=self.__initial(sid), mode=self.mode)

        overmax = (len(CHBox().load()) >= MAX_JOB_NUMBER)
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "job.html"),
            self.KEY_TEMPLATE_DIC: {
                "mode": self.mode,
                "overmax": overmax,
                "form": form,
            },
        }

    def view_apply(self, request, sid=None):
        form = JobForm(data=request.POST, mode=self.mode)

        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                chbox = CHBox()
                loginid, portal = form.cleaned_data["boxid"].split("@")[:2]
                job = {
                    "sid": form.cleaned_data["sid"],
                    "enable": form.cleaned_data["enable"],
                    #"downloadinterval": form.cleaned_data["interval"],
                    "downloadinterval": form.cleaned_data["interval"] if form.cleaned_data["download"] else 0,
                    "uploadmaxfilesize": form.cleaned_data["uploadsize"] or 0,
                    "share": {
                        "name": form.cleaned_data["sharename"],
                    },
                    "chbox": {
                        "portal": portal,
                        "loginid": loginid,
                        #"portal": form.cleaned_data["portal"],
                        #"loginid": form.cleaned_data["loginid"],
                        "passwd": form.cleaned_data["password"],
                        "folder": {
                            "path": form.cleaned_data["folder"],
                        },
                    },
                }
                p.result((chbox.modify if self.mode == "modify" else chbox.add)(job))
                self.set_message(chbox.get_message())
                if p.result():
                    # sharenameのchoicesが変わるかもしれないため
                    form = JobForm(initial=request.POST, mode=self.mode)

        overmax = (len(CHBox().load()) >= MAX_JOB_NUMBER)
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "job.html"),
            self.KEY_TEMPLATE_DIC: {
                "mode": self.mode,
                "overmax": overmax,
                "form": form,
            },
        }


class AddView(JobView):

    mode = "add"


class ModifyView(JobView):

    mode = "modify"


class DataListView(RootView):

    MAX = 10

    def view(self, request):
        datalist = []
        try:
            cleaned_data = self.__cleaned_data(request.POST)
            if cleaned_data is None:
                raise
            cloud = CHBox()
            loginid, portal = cleaned_data["boxid"].split("@")[:2]
            for folder in cloud.list(
                    #portal=cleaned_data["portal"],
                    #loginid=cleaned_data["loginid"],
                    portal=portal,
                    loginid=loginid,
                    passwd=cleaned_data["password"]):
                try:
                    if re.search(
                                r"%s" % cleaned_data["folder"],
                                unicode(folder["path"], 'utf-8')):
                        datalist.append({
                            "label": folder["path"],
                            "value": folder["path"],
                        })
                except:
                    pass
            datalist.sort(key=lambda data: data["label"])
            #datalist = datalist[:self.MAX]
            from fal_info import Results
            result = int(cloud.get_result())
            if cleaned_data["folder"] == "":
                if result == int(Results.OK):
                    if len(datalist) == 0:
                        datalist.append({
                                "label": _(u"CHBox: Not Found (Folder)"),
                                "value": "",
                        })
                else:
                    datalist.append({
                            "label": _(u"CHBox: Failed to get folder."),
                            "value": "",
                    })
        except:
            pass
        return {
            self.KEY_RESPONSE: HttpResponse(
                    simplejson.dumps(datalist),
                    mimetype="application/json;charset=UTF-8;"),
        }

    def __cleaned_data(self, data):
        try:
            form = JobForm()
            cleaned_data = {}
            #for name in ["portal", "loginid", "password", "folder"]:
            for name in ["boxid", "password", "folder"]:
                cleaned_data[name] = form.fields[name].clean(data[name])
            return cleaned_data
        except:
            return None
